@@ -38,8 +38,8 @@ static gchar *DefaultHeight = "ColorSel|GtkWindow::default_height";
 static gchar *Shrink = "ColorSelDialog|GtkWindow::allow_shrink";
 static gchar *Grow = "ColorSelDialog|GtkWindow::allow_grow";
 static gchar *AutoShrink = "ColorSelDialog|GtkWindow::auto_shrink";
-static gchar *WMName = "ColorSelDialog|GtkWindow::wmclass_name";
-static gchar *WMClass = "ColorSelDialog|GtkWindow::wmclass_class";
+static gchar *IconName = "ColorSelDialog|GtkWindow::icon_name";
+static gchar *FocusOnMap = "ColorSelDialog|GtkWindow::focus_on_map";
 
 static gchar *Resizable = "ColorSelDialog|GtkWindow::resizable";
 static gchar *DestroyWithParent = "ColorSelDialog|GtkWindow::destroy_with_parent";
@@ -113,7 +113,7 @@ gb_color_selection_dialog_create_properties (GtkWidget * widget,
 					Title, Type, Position, Modal,
 					DefaultWidth, DefaultHeight,
 					Shrink, Grow, AutoShrink,
-					WMName, WMClass,
+					IconName, FocusOnMap,
 					Resizable, DestroyWithParent, Icon,
 					Role, TypeHint, SkipTaskbar,
 					SkipPager, Decorated, Gravity);
@@ -139,7 +139,7 @@ gb_color_selection_dialog_get_properties (GtkWidget * widget,
 				     Title, Type, Position, Modal,
 				     DefaultWidth, DefaultHeight,
 				     Shrink, Grow, AutoShrink,
-				     WMName, WMClass,
+				     IconName, FocusOnMap,
 				     Resizable, DestroyWithParent, Icon,
 				     Role, TypeHint, SkipTaskbar,
 				     SkipPager, Decorated, Gravity);
@@ -165,7 +165,7 @@ gb_color_selection_dialog_set_properties (GtkWidget * widget,
 				     Title, Type, Position, Modal,
 				     DefaultWidth, DefaultHeight,
 				     Shrink, Grow, AutoShrink,
-				     WMName, WMClass,
+				     IconName, FocusOnMap,
 				     Resizable, DestroyWithParent, Icon,
 				     Role, TypeHint, SkipTaskbar,
 				     SkipPager, Decorated, Gravity);
@@ -222,7 +222,7 @@ gb_color_selection_dialog_write_source (GtkWidget * widget,
 				   NULL, Type, Position, Modal,
 				   DefaultWidth, DefaultHeight,
 				   Shrink, Grow, AutoShrink,
-				   WMName, WMClass,
+				   IconName, FocusOnMap,
 				   Resizable, DestroyWithParent, Icon,
 				   Role, TypeHint, SkipTaskbar,
 				   SkipPager, Decorated, Gravity);
