@@ -23,6 +23,7 @@
 #include <gtk/gtkcombo.h>
 #include <gtk/gtkcombobox.h>
 #include <gtk/gtkentry.h>
+#include <gtk/gtkexpander.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtklist.h>
 #include <gtk/gtklistitem.h>
@@ -51,6 +52,11 @@ static gchar *XPad = "Label|GtkMisc::xpad";
 static gchar *YPad = "Label|GtkMisc::ypad";
 static gchar *Selectable = "GtkLabel::selectable";
 
+static gchar *Ellipsize = "GtkLabel::ellipsize";
+static gchar *WidthChars = "GtkLabel::width_chars";
+static gchar *SingleLineMode = "GtkLabel::single_line_mode";
+static gchar *Angle = "GtkLabel::angle";
+
 static gchar *FocusTarget = "GtkLabel::mnemonic_widget";
 
 
@@ -77,6 +83,29 @@ static const gchar *GbJustifySymbols[] =
   "GTK_JUSTIFY_FILL"
 };
 
+static const gchar *GbEllipsizeChoices[] =
+{
+  "None",
+  "Start",
+  "Middle",
+  "End",
+  NULL
+};
+static const gint GbEllipsizeValues[] =
+{
+  PANGO_ELLIPSIZE_NONE,
+  PANGO_ELLIPSIZE_START,
+  PANGO_ELLIPSIZE_MIDDLE,
+  PANGO_ELLIPSIZE_END
+};
+static const gchar *GbEllipsizeSymbols[] =
+{
+  "PANGO_ELLIPSIZE_NONE",
+  "PANGO_ELLIPSIZE_START",
+  "PANGO_ELLIPSIZE_MIDDLE",
+  "PANGO_ELLIPSIZE_END"
+};
+
 
 static void gb_label_get_focus_targets (GtkWidget * widget,
 					GList ** focus_targets);
@@ -128,7 +157,11 @@ gb_label_create_standard_properties (GtkWidget * widget,
 				     const char *yalign_p,
 				     const char *xpad_p,
 				     const char *ypad_p,
-				     const char *focus_target_p)
+				     const char *focus_target_p,
+				     const char *ellipsize_p,
+				     const char *width_chars_p,
+				     const char *single_line_mode_p,
+				     const char *angle_p)
 {
   GtkWidget *combo;
 
@@ -160,6 +193,18 @@ gb_label_create_standard_properties (GtkWidget * widget,
   combo = property_get_value_widget (focus_target_p);
   gtk_editable_set_editable (GTK_EDITABLE (GTK_COMBO (combo)->entry), FALSE);
   /*gtk_combo_set_value_in_list (GTK_COMBO (combo), TRUE, TRUE);*/
+
+  property_add_choice (ellipsize_p, _("Ellipsize:"),
+		       _("How to ellipsize the string"),
+		       GbEllipsizeChoices);
+  property_add_int_range (width_chars_p, _("Width in Chars:"),
+			  _("The width of the label in characters"),
+			  -1, 1000, 1, 10, 1);
+  property_add_bool (single_line_mode_p, _("Single Line Mode:"),
+		     _("If the label is only given enough height for a single line"));
+  property_add_float_range (angle_p, _("Angle:"),
+			    _("The angle of the label text"),
+			    0, 360, 1, 10, 1, 2);
 }
 
 
@@ -173,7 +218,8 @@ gb_label_create_properties (GtkWidget * widget, GbWidgetCreateArgData * data)
 				       Label, UseUnderline, UseMarkup,
 				       Justify, Wrap, Selectable,
 				       XAlign, YAlign, XPad, YPad,
-				       FocusTarget);
+				       FocusTarget, Ellipsize, WidthChars,
+				       SingleLineMode, Angle);
 }
 
 
@@ -190,6 +236,13 @@ gb_label_find_mnemonic_widget (GtkWidget *widget)
 
   while (parent)
     {
+      if (GTK_IS_EXPANDER (parent))
+	{
+	  if (gtk_expander_get_label_widget (GTK_EXPANDER (parent)) == widget)
+	    return TRUE;
+	  else
+	    return FALSE;
+	}
       if (GTK_WIDGET_GET_CLASS (parent)->activate_signal)
 	{
 	  return TRUE;
@@ -219,10 +272,17 @@ gb_label_get_standard_properties (GtkWidget * widget,
 				  const char *yalign_p,
 				  const char *xpad_p,
 				  const char *ypad_p,
-				  const char *focus_target_p)
+				  const char *focus_target_p,
+				  const char *ellipsize_p,
+				  const char *width_chars_p,
+				  const char *single_line_mode_p,
+				  const char *angle_p)
 {
   const gchar *label_text;
-  gint i;
+  PangoEllipsizeMode ellipsize_mode;
+  gint i, width_chars;
+  gboolean single_line_mode;
+  gfloat angle;
 
   label_text = gtk_label_get_label (GTK_LABEL (widget));
   gb_widget_output_translatable_text (data, label_p, label_text);
@@ -325,6 +385,22 @@ gb_label_get_standard_properties (GtkWidget * widget,
 	  property_set_visible (focus_target_p, FALSE);
 	}
     }
+
+  ellipsize_mode = gtk_label_get_ellipsize (GTK_LABEL (widget));
+  for (i = 0; i < sizeof (GbEllipsizeValues) / sizeof (GbEllipsizeValues[0]); i++)
+    {
+      if (GbEllipsizeValues[i] == ellipsize_mode)
+	gb_widget_output_choice (data, ellipsize_p, i, GbEllipsizeSymbols[i]);
+    }
+
+  width_chars = gtk_label_get_width_chars (GTK_LABEL (widget));
+  gb_widget_output_int (data, width_chars_p, width_chars);
+
+  single_line_mode = gtk_label_get_single_line_mode (GTK_LABEL (widget));
+  gb_widget_output_bool (data, single_line_mode_p, single_line_mode);
+
+  angle = gtk_label_get_angle (GTK_LABEL (widget));
+  gb_widget_output_float (data, angle_p, angle);
 }
 
 
@@ -339,7 +415,8 @@ gb_label_get_properties (GtkWidget * widget, GbWidgetGetArgData * data)
 				    Label, UseUnderline, UseMarkup,
 				    Justify, Wrap, Selectable,
 				    XAlign, YAlign, XPad, YPad,
-				    FocusTarget);
+				    FocusTarget, Ellipsize, WidthChars,
+				    SingleLineMode, Angle);
 }
 
 
@@ -386,14 +463,18 @@ gb_label_set_standard_properties (GtkWidget * widget,
 				  const char *yalign_p,
 				  const char *xpad_p,
 				  const char *ypad_p,
-				  const char *focus_target_p)
+				  const char *focus_target_p,
+				  const char *ellipsize_p,
+				  const char *width_chars_p,
+				  const char *single_line_mode_p,
+				  const char *angle_p)
 {
-  gchar *label, *justify, *accel_target;
+  gchar *label, *justify, *accel_target, *ellipsize_mode;
   const gchar *label_text;
-  gfloat xalign, yalign;
-  gint xpad, ypad, i;
+  gfloat xalign, yalign, angle;
+  gint xpad, ypad, i, width_chars;
   gboolean wrap, selectable, set_alignment = FALSE, set_padding = FALSE;
-  gboolean use_underline, use_markup;
+  gboolean use_underline, use_markup, single_line_mode;
   gboolean set_label = FALSE;
 
   use_underline = gb_widget_input_bool (data, use_underline_p);
@@ -538,6 +619,34 @@ gb_label_set_standard_properties (GtkWidget * widget,
 				    accel_target ? g_free : NULL);
 	}
     }
+
+  ellipsize_mode = gb_widget_input_choice (data, ellipsize_p);
+  if (data->apply)
+    {
+      for (i = 0; i < sizeof (GbEllipsizeValues) / sizeof (GbEllipsizeValues[0]);
+	   i++)
+	{
+	  if (!strcmp (ellipsize_mode, GbEllipsizeChoices[i])
+	      || !strcmp (ellipsize_mode, GbEllipsizeSymbols[i]))
+	    {
+	      gtk_label_set_ellipsize (GTK_LABEL (widget),
+				       GbEllipsizeValues[i]);
+	      break;
+	    }
+	}
+    }
+
+  width_chars = gb_widget_input_int (data, width_chars_p);
+  if (data->apply)
+    gtk_label_set_width_chars (GTK_LABEL (widget), width_chars);
+
+  single_line_mode = gb_widget_input_bool (data, single_line_mode_p);
+  if (data->apply)
+    gtk_label_set_single_line_mode (GTK_LABEL (widget), single_line_mode);
+
+  angle = gb_widget_input_float (data, angle_p);
+  if (data->apply)
+    gtk_label_set_angle (GTK_LABEL (widget), angle);
 }
 
 
@@ -552,7 +661,8 @@ gb_label_set_properties (GtkWidget * widget, GbWidgetSetArgData * data)
 				    Label, UseUnderline, UseMarkup,
 				    Justify, Wrap, Selectable,
 				    XAlign, YAlign, XPad, YPad,
-				    FocusTarget);
+				    FocusTarget, Ellipsize, WidthChars,
+				    SingleLineMode, Angle);
 }
 
 
@@ -583,10 +693,17 @@ gb_label_write_standard_source (GtkWidget * widget,
 				const char *yalign_p,
 				const char *xpad_p,
 				const char *ypad_p,
-				const char *focus_target_p)
+				const char *focus_target_p,
+				const char *ellipsize_p,
+				const char *width_chars_p,
+				const char *single_line_mode_p,
+				const char *angle_p)
 {
   GtkWidget *accel_target = NULL;
-  gint i;
+  PangoEllipsizeMode ellipsize_mode;
+  gint i, width_chars;
+  gboolean single_line_mode;
+  gfloat angle;
 
   if (gtk_object_get_data (GTK_OBJECT (widget), use_markup_p))
     source_add (data, "  gtk_label_set_use_markup (GTK_LABEL (%s), TRUE);\n",
@@ -647,6 +764,44 @@ gb_label_write_standard_source (GtkWidget * widget,
   if (GTK_MISC (widget)->xpad != 0 || GTK_MISC (widget)->ypad != 0)
     source_add (data, "  gtk_misc_set_padding (GTK_MISC (%s), %i, %i);\n",
 	     data->wname, GTK_MISC (widget)->xpad, GTK_MISC (widget)->ypad);
+
+  ellipsize_mode = gtk_label_get_ellipsize (GTK_LABEL (widget));
+  if (ellipsize_mode != PANGO_ELLIPSIZE_NONE)
+    {
+      for (i = 0;
+	   i < sizeof (GbEllipsizeValues) / sizeof (GbEllipsizeValues[0]);
+	   i++)
+	{
+	  if (GbEllipsizeValues[i] == ellipsize_mode)
+	    source_add (data,
+			"  gtk_label_set_ellipsize (GTK_LABEL (%s), %s);\n",
+			data->wname, GbEllipsizeSymbols[i]);
+	}
+    }
+
+  width_chars = gtk_label_get_width_chars (GTK_LABEL (widget));
+  if (width_chars != -1)
+    {
+      source_add (data,
+		  "  gtk_label_set_width_chars (GTK_LABEL (%s), %i);\n",
+		  data->wname, width_chars);
+    }
+
+  single_line_mode = gtk_label_get_single_line_mode (GTK_LABEL (widget));
+  if (single_line_mode)
+    {
+      source_add (data,
+		  "  gtk_label_set_single_line_mode (GTK_LABEL (%s), TRUE);\n",
+		  data->wname);
+    }
+
+  angle = gtk_label_get_angle (GTK_LABEL (widget));
+  if (angle > GLADE_EPSILON)
+    {
+      source_add (data,
+		  "  gtk_label_set_angle (GTK_LABEL (%s), %g);\n",
+		  data->wname, angle);
+    }
 }
 
 
@@ -658,18 +813,29 @@ gb_label_write_standard_source (GtkWidget * widget,
 static void
 gb_label_write_source (GtkWidget * widget, GbWidgetWriteSourceData * data)
 {
+  gboolean translatable, context;
+  gchar *comments;
+
   if (data->create_widget)
     {
       const gchar *label_text = gtk_label_get_label (GTK_LABEL (widget));
 
+      glade_util_get_translation_properties (widget, Label, &translatable,
+					     &comments, &context);
+      source_add_translator_comments (data, translatable, comments);
+
       if (gtk_label_get_use_underline (GTK_LABEL (widget)))
 	source_add (data, "  %s = gtk_label_new_with_mnemonic (%s);\n",
 		    data->wname,
-		    source_make_string (label_text, data->use_gettext));
+		    source_make_string_full (label_text,
+					     data->use_gettext && translatable,
+					     context));
       else
 	source_add (data, "  %s = gtk_label_new (%s);\n",
 		    data->wname,
-		    source_make_string (label_text, data->use_gettext));
+		    source_make_string_full (label_text,
+					     data->use_gettext && translatable,
+					     context));
     }
 
   gb_widget_write_standard_source (widget, data);
@@ -678,7 +844,8 @@ gb_label_write_source (GtkWidget * widget, GbWidgetWriteSourceData * data)
 				  Label, UseUnderline, UseMarkup,
 				  Justify, Wrap, Selectable,
 				  XAlign, YAlign, XPad, YPad,
-				  FocusTarget);
+				  FocusTarget, Ellipsize, WidthChars,
+				  SingleLineMode, Angle);
 }
 
 
