@@ -60,7 +60,7 @@ find_parents_group (GtkWidget * widget, GSList ** group)
 
   if (GTK_IS_RADIO_MENU_ITEM (widget))
     {
-      *group = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (widget));
+      *group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (widget));
     }
 }
 
@@ -78,7 +78,9 @@ gb_radio_menu_item_new (GbWidgetNewData * data)
   GtkWidget *new_widget;
   GSList *group_list = NULL;
 
-  if (data->parent)
+  /* When creating a radiomenuitem we try to place it in the same group as
+     other radiomenuitems with the same parent. */
+  if (data->parent && data->action == GB_CREATING)
     gtk_container_foreach (GTK_CONTAINER (data->parent),
 			   (GtkCallback) find_parents_group, &group_list);
 
@@ -104,6 +106,8 @@ gb_radio_menu_item_create_properties (GtkWidget * widget, GbWidgetCreateArgData
   property_add_combo (Group, _("Group:"),
 		      _("The radio menu item group (the default is all radio menu items with the same parent)"),
 		      NULL);
+  /* The Group property is only used for loading and saving at present. */
+  property_set_visible (Group, FALSE);
 }
 
 
@@ -182,7 +186,9 @@ find_group_widget (GtkWidget *widget, GladeFindGroupWidgetData *find_data)
     {
       if (!strcmp (gtk_widget_get_name (widget), find_data->name))
 	{
+#if 0
 	  g_print ("Found widget: %s\n", find_data->name);
+#endif
 	  find_data->found_widget = widget;
 	  return;
 	}
@@ -228,7 +234,7 @@ gb_radio_menu_item_set_properties (GtkWidget * widget, GbWidgetSetArgData * data
 	{
 	  GSList *old_group, *new_group = NULL;
 
-	  old_group = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (widget));
+	  old_group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (widget));
 
 	  if (group_name && group_name[0] == '\0')
 	    group_name = NULL;
@@ -250,8 +256,10 @@ gb_radio_menu_item_set_properties (GtkWidget * widget, GbWidgetSetArgData * data
 
 	  if (new_group != old_group)
 	    {
+#if 0
 	      g_print ("##### setting radio group: %s\n",
 		       group_name ? group_name : "NULL");
+#endif
 	      gtk_radio_menu_item_set_group (GTK_RADIO_MENU_ITEM (widget),
 					     new_group);
 	    }
@@ -290,6 +298,8 @@ gb_radio_menu_item_write_source (GtkWidget * widget, GbWidgetWriteSourceData *
   GtkWidget *group_widget;
   GtkWidget *child = GTK_BIN (widget)->child;
   gchar buffer[256], *group_name, *label_text;
+  gboolean translatable, context;
+  gchar *comments;
 
 #ifdef USE_GNOME
   if (data->project->gnome_support)
@@ -322,10 +332,15 @@ gb_radio_menu_item_write_source (GtkWidget * widget, GbWidgetWriteSourceData *
       if (child && GTK_IS_LABEL (child) && !GB_IS_GB_WIDGET (child))
 	{
 	  label_text = glade_util_get_label_text (child);
+
+	  glade_util_get_translation_properties (widget, Label, &translatable,
+						 &comments, &context);
+	  source_add_translator_comments (data, translatable, comments);
+
 	  source_add (data,
 		      "  %s = gtk_radio_menu_item_new_with_mnemonic (%s_group, %s);\n",
 		      data->wname, group_name,
-		      source_make_string (label_text, data->use_gettext));
+		      source_make_string_full (label_text, data->use_gettext && translatable, context));
 	  g_free (label_text);
 	}
       else
@@ -334,7 +349,7 @@ gb_radio_menu_item_write_source (GtkWidget * widget, GbWidgetWriteSourceData *
 		      data->wname, group_name);
 	}
       source_add (data,
-      "  %s_group = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (%s));\n",
+      "  %s_group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (%s));\n",
 		  group_name, data->wname);
       g_free (group_name);
     }
