@@ -25,7 +25,7 @@
 
 /* This file includes some common routines for standard window properties
    such as title, type, position, default width & height, shrink, grow,
-   auto_shrink, wmname & wmclass. Other widgets can call these functions just
+   auto_shrink & icon. Other widgets can call these functions just
    by supplying the property names. Note that for most properties we use the
    widget's own property names as keys when storing values in the objects
    datalist, but for default width & height we use the windows own names since
@@ -46,8 +46,8 @@ static gchar *Position = "GtkWindow::window_position";
 static gchar *Modal = "GtkWindow::modal";
 static gchar *DefaultWidth = "GtkWindow::default_width";
 static gchar *DefaultHeight = "GtkWindow::default_height";
-static gchar *WMName = "GtkWindow::wmclass_name";
-static gchar *WMClass = "GtkWindow::wmclass_class";
+static gchar *IconName = "GtkWindow::icon_name";
+static gchar *FocusOnMap = "GtkWindow::focus_on_map";
 static gchar *Resizable = "GtkWindow::resizable";
 static gchar *DestroyWithParent = "GtkWindow::destroy_with_parent";
 static gchar *Icon = "GtkWindow::icon";
@@ -207,7 +207,7 @@ gb_window_create_properties (GtkWidget * widget, GbWidgetCreateArgData * data)
 					Title, Type, Position, Modal,
 					DefaultWidth, DefaultHeight,
 					Shrink, Grow, NULL,
-					WMName, WMClass,
+					IconName, FocusOnMap,
 					Resizable, DestroyWithParent, Icon,
 					Role, TypeHint, SkipTaskbar,
 					SkipPager, Decorated, Gravity);
@@ -226,8 +226,8 @@ gb_window_create_standard_properties (GtkWidget * widget,
 				      gchar *shrink_p,
 				      gchar *grow_p,
 				      gchar *auto_shrink_p,
-				      gchar *wmname_p,
-				      gchar *wmclass_p,
+				      gchar *icon_name_p,
+				      gchar *focus_on_map_p,
 				      gchar *resizable_p,
 				      gchar *destroy_with_parent_p,
 				      gchar *icon_p,
@@ -287,16 +287,6 @@ gb_window_create_standard_properties (GtkWidget * widget,
     }
 #endif
 
-#if 0
-  /* These aren't necessary, and have been used incorrectly for ages. */
-  if (wmname_p)
-    property_add_string (wmname_p, _("WM Name:"),
-			 _("The name to pass to the window manager"));
-  if (wmclass_p)
-    property_add_string (wmclass_p, _("WM Class:"),
-			 _("The class name to pass to the window manager"));
-#endif
-
   if (destroy_with_parent_p)
     property_add_bool (destroy_with_parent_p, _("Auto-Destroy:"),
 		       _("If the window is destroyed when its transient parent is destroyed"));
@@ -305,6 +295,10 @@ gb_window_create_standard_properties (GtkWidget * widget,
     property_add_filename (icon_p, _("Icon:"),
 			   _("The icon for this window"));
 
+  if (icon_name_p)
+    property_add_named_icon (icon_name_p, _("Named Icon:"),
+			     _("The named icon to use"));
+
   if (role_p)
     property_add_string (role_p, _("Role:"), _("A unique identifier for the window to be used when restoring a session"));
 
@@ -325,6 +319,8 @@ gb_window_create_standard_properties (GtkWidget * widget,
 			 _("The reference point to use when the window coordinates are set"),
 			 GbGravityChoices);
 
+  if (focus_on_map_p)
+    property_add_bool (focus_on_map_p, _("Focus On Map:"), _("If the window should receive the input focus when it is mapped"));
 }
 
 
@@ -407,7 +403,7 @@ gb_window_get_properties (GtkWidget * widget, GbWidgetGetArgData * data)
 				     Title, Type, Position, Modal,
 				     DefaultWidth, DefaultHeight,
 				     Shrink, Grow, NULL,
-				     WMName, WMClass,
+				     IconName, FocusOnMap,
 				     Resizable, DestroyWithParent, Icon,
 				     Role, TypeHint, SkipTaskbar,
 				     SkipPager, Decorated, Gravity);
@@ -426,8 +422,8 @@ gb_window_get_standard_properties (GtkWidget * widget,
 				   gchar *shrink_p,
 				   gchar *grow_p,
 				   gchar *auto_shrink_p,
-				   gchar *wmname_p,
-				   gchar *wmclass_p,
+				   gchar *icon_name_p,
+				   gchar *focus_on_map_p,
 				   gchar *resizable_p,
 				   gchar *destroy_with_parent_p,
 				   gchar *icon_p,
@@ -495,20 +491,6 @@ gb_window_get_standard_properties (GtkWidget * widget,
     }
 #endif
 
-  /* These are stored in the object hash since we can't set them after the
-     window is realized. */
-#if 0
-  /* These aren't necessary, and have been used incorrectly for ages. */
-  if (wmname_p)
-    gb_widget_output_string (data, wmname_p,
-			     gtk_object_get_data (GTK_OBJECT (widget),
-						  wmname_p));
-  if (wmclass_p)
-    gb_widget_output_string (data, wmclass_p,
-			     gtk_object_get_data (GTK_OBJECT (widget),
-						  wmclass_p));
-#endif
-
   if (resizable_p)
     gb_widget_output_bool (data, resizable_p,
 			   gtk_window_get_resizable (GTK_WINDOW (widget)));
@@ -522,6 +504,10 @@ gb_window_get_standard_properties (GtkWidget * widget,
 				      gtk_object_get_data (GTK_OBJECT (widget),
 							   GladeWindowIconKey));
 
+  if (icon_name_p)
+      gb_widget_output_named_icon (data, icon_name_p,
+				   gtk_window_get_icon_name (GTK_WINDOW (widget)));
+
   if (role_p)
     gb_widget_output_string (data, role_p,
 			     gtk_window_get_role (GTK_WINDOW (widget)));
@@ -565,6 +551,12 @@ gb_window_get_standard_properties (GtkWidget * widget,
       gb_widget_output_choice (data, gravity_p, gravity,
 			       GbGravitySymbols[gravity]);
     }
+
+  if (focus_on_map_p)
+    {
+      gb_widget_output_bool (data, focus_on_map_p,
+			     gtk_object_get_data (GTK_OBJECT (widget), focus_on_map_p) == NULL ? TRUE : FALSE);
+    }
 }
 
 
@@ -580,13 +572,27 @@ gb_window_set_properties (GtkWidget * widget, GbWidgetSetArgData * data)
 				     Title, Type, Position, Modal,
 				     DefaultWidth, DefaultHeight,
 				     Shrink, Grow, NULL,
-				     WMName, WMClass,
+				     IconName, FocusOnMap,
 				     Resizable, DestroyWithParent, Icon,
 				     Role, TypeHint, SkipTaskbar,
 				     SkipPager, Decorated, Gravity);
 }
 
 
+static void
+gb_window_clear_pixmap (GtkWidget *widget, GladeProject *project)
+{
+  gchar *old_icon;
+
+  /* Remove the old icon stored in the widget data, and remove the
+     pixmap from the project, if necessary. */
+  old_icon = gtk_object_get_data (GTK_OBJECT (widget), GladeWindowIconKey);
+  glade_project_remove_pixmap (project, old_icon);
+
+  gtk_object_set_data (GTK_OBJECT (widget), GladeWindowIconKey, NULL);
+}
+
+
 void
 gb_window_set_standard_properties (GtkWidget * widget,
 				   GbWidgetSetArgData * data,
@@ -599,8 +605,8 @@ gb_window_set_standard_properties (GtkWidget * widget,
 				   gchar *shrink_p,
 				   gchar *grow_p,
 				   gchar *auto_shrink_p,
-				   gchar *wmname_p,
-				   gchar *wmclass_p,
+				   gchar *icon_name_p,
+				   gchar *focus_on_map_p,
 				   gchar *resizable_p,
 				   gchar *destroy_with_parent_p,
 				   gchar *icon_p,
@@ -617,9 +623,6 @@ gb_window_set_standard_properties (GtkWidget * widget,
   gboolean resizable, destroy_with_parent;
   gboolean decorated, skip_taskbar, skip_pager;
   gchar *type_hint, *gravity;
-#if 0
-  gchar *wmname, *wmclass;
-#endif
 
   if (title_p)
     {
@@ -730,34 +733,6 @@ gb_window_set_standard_properties (GtkWidget * widget,
     }
 #endif
 
-#if 0
-  /* These aren't necessary, and have been used incorrectly for ages. */
-  if (wmname_p)
-    {
-      wmname = gb_widget_input_string (data, wmname_p);
-      if (wmname && wmname[0] == '\0')
-	wmname = NULL;
-      if (data->apply)
-	{
-	  gtk_object_set_data_full (GTK_OBJECT (widget), wmname_p,
-				    g_strdup (wmname), wmname ? g_free : NULL);
-	}
-    }
-
-  if (wmclass_p)
-    {
-      wmclass = gb_widget_input_string (data, wmclass_p);
-      if (wmclass && wmclass[0] == '\0')
-	wmclass = NULL;
-      if (data->apply)
-	{
-	  gtk_object_set_data_full (GTK_OBJECT (widget), wmclass_p,
-				    g_strdup (wmclass),
-				    wmclass ? g_free : NULL);
-	}
-    }
-#endif
-
   if (resizable_p)
     {
       resizable = gb_widget_input_bool (data, resizable_p);
@@ -778,14 +753,12 @@ gb_window_set_standard_properties (GtkWidget * widget,
       char *filename = gb_widget_input_pixmap_filename (data, icon_p);
       if (data->apply)
 	{
-	  char *old_filename;
-
 	  if (filename && filename[0] == '\0')
 	    filename = NULL;
 
-	  /* Remove the old pixmap from the project. */
-	  old_filename = gtk_object_get_data (GTK_OBJECT (widget), GladeWindowIconKey);
-	  glade_project_remove_pixmap (data->project, old_filename);
+	  /* Remove the old icon stored in the widget data, and remove the
+	     pixmap from the project, if necessary. */
+	  gb_window_clear_pixmap (widget, data->project);
 
 	  gtk_object_set_data_full (GTK_OBJECT (widget), GladeWindowIconKey,
 				    g_strdup (filename),
@@ -800,11 +773,40 @@ gb_window_set_standard_properties (GtkWidget * widget,
 	      if (pixbuf)
 		gdk_pixbuf_unref (pixbuf);
 	    }
+
+	  if (property_get_widget () == widget && icon_name_p)
+	    {
+	      /* Clear the IconName property. */
+	      property_set_auto_apply (FALSE);
+	      property_set_named_icon (icon_name_p, NULL);
+	      property_set_auto_apply (TRUE);
+	    }
 	}
       if (data->action == GB_LOADING)
 	g_free (filename);
     }
 
+  if (icon_name_p)
+    {
+      gchar *icon_name = gb_widget_input_named_icon (data, IconName);
+
+      if (data->apply)
+	{
+	  /* Clear any stock icon or icon from a file. */
+	  gb_window_clear_pixmap (widget, data->project);
+
+	  gtk_window_set_icon (GTK_WINDOW (widget), NULL);
+	  gtk_window_set_icon_name (GTK_WINDOW (widget), icon_name);
+
+	  if (property_get_widget () == widget && icon_p)
+	    {
+	      /* Clear the Icon property. */
+	      property_set_auto_apply (FALSE);
+	      property_set_filename (icon_p, NULL);
+	      property_set_auto_apply (TRUE);
+	    }
+	}
+    }
 
   if (role_p)
     {
@@ -876,6 +878,16 @@ gb_window_set_standard_properties (GtkWidget * widget,
 	    }
 	}
     }
+
+  if (focus_on_map_p)
+    {
+      gboolean focus_on_map = gb_widget_input_bool (data, focus_on_map_p);
+      if (data->apply)
+	{
+	  gtk_object_set_data (GTK_OBJECT (widget), focus_on_map_p,
+			       focus_on_map ? NULL : "FALSE");
+	}
+    }
 }
 
 
@@ -919,7 +931,7 @@ gb_window_write_source (GtkWidget * widget, GbWidgetWriteSourceData * data)
 				   Title, NULL, Position, Modal,
 				   DefaultWidth, DefaultHeight,
 				   Shrink, Grow, NULL,
-				   WMName, WMClass,
+				   IconName, FocusOnMap,
 				   Resizable, DestroyWithParent, Icon,
 				   Role, TypeHint, SkipTaskbar,
 				   SkipPager, Decorated, Gravity);
@@ -938,8 +950,8 @@ gb_window_write_standard_source (GtkWidget * widget,
 				 gchar *shrink_p,
 				 gchar *grow_p,
 				 gchar *auto_shrink_p,
-				 gchar *wmname_p,
-				 gchar *wmclass_p,
+				 gchar *icon_name_p,
+				 gchar *focus_on_map_p,
 				 gchar *resizable_p,
 				 gchar *destroy_with_parent_p,
 				 gchar *icon_p,
@@ -953,9 +965,6 @@ gb_window_write_standard_source (GtkWidget * widget,
   gint type, position, default_width, default_height, type_hint, gravity;
   gboolean translatable, context;
   gchar *comments;
-#if 0
-  gchar *wmname, *wmclass;
-#endif
 
   if (title_p)
     {
@@ -1039,24 +1048,6 @@ gb_window_write_standard_source (GtkWidget * widget,
     }
 #endif
 
-#if 0
-  /* These aren't necessary, and have been used incorrectly for ages. */
-  if (wmname_p && wmclass_p)
-    {
-      wmname = gtk_object_get_data (GTK_OBJECT (widget), wmname_p);
-      wmclass = gtk_object_get_data (GTK_OBJECT (widget), wmclass_p);
-      if (wmname || wmclass)
-	{
-	  source_add (data,
-		      "  gtk_window_set_wmclass (GTK_WINDOW (%s), %s,",
-		      data->wname,
-		      wmname ? source_make_string (wmname, FALSE) : "\"\"");
-	  source_add (data, " %s);\n",
-		      wmclass ? source_make_string (wmclass, FALSE) : "\"\"");
-	}
-    }
-#endif
-
   if (resizable_p)
     {
       if (!gtk_window_get_resizable (GTK_WINDOW (widget)))
@@ -1096,6 +1087,17 @@ gb_window_write_standard_source (GtkWidget * widget,
 	}
     }
 
+  if (icon_p)
+    {
+      const gchar *icon_name = gtk_window_get_icon_name (GTK_WINDOW (widget));
+      if (icon_name && *icon_name)
+	{
+	  source_add (data,
+		      "  gtk_window_set_icon_name (GTK_WINDOW (%s), \"%s\");\n",
+		      data->wname, icon_name);
+	}
+    }
+
   if (role_p)
     {
       const gchar *role = gtk_window_get_role (GTK_WINDOW (widget));
@@ -1156,17 +1158,23 @@ gb_window_write_standard_source (GtkWidget * widget,
 		      data->wname, GbGravitySymbols[gravity]);
 	}
     }
+
+  if (focus_on_map_p)
+    {
+      if (gtk_object_get_data (GTK_OBJECT (widget), focus_on_map_p))
+	{
+	  source_add (data,
+		      "  gtk_window_set_focus_on_map (GTK_WINDOW (%s), FALSE);\n",
+		      data->wname);
+	}
+    }
 }
 
 
 void
 gb_window_destroy (GtkWidget * widget, GbWidgetDestroyData * data)
 {
-  gchar *filename;
-
-  /* glade_project_remove_pixmap() will just ignore it if it is NULL. */
-  filename = gtk_object_get_data (GTK_OBJECT (widget), GladeWindowIconKey);
-  glade_project_remove_pixmap (data->project, filename);
+  gb_window_clear_pixmap (widget, data->project);
 }
 
 
