@@ -18,6 +18,8 @@
 
 #include "gladeconfig.h"
 
+#include <string.h>
+
 #include <gtk/gtkhbox.h>
 #include <gtk/gtkhseparator.h>
 #include <gtk/gtklabel.h>
@@ -119,6 +121,31 @@ glade_palette_class_init (GladePaletteClass * klass)
 }
 
 
+/* Ensure the correct button is depressed when the page is switched with
+   keyboard accelerators. */
+static void
+on_notebook_switch_page (GtkNotebook *notebook, GtkNotebookPage *page,
+			 guint page_num, GladePalette *palette)
+{
+  GSList *elem;
+  gint last_page;
+
+  last_page = g_list_length (notebook->children) - 1;
+
+  for (elem = palette->section_button_group; elem; elem = elem->next)
+    {
+      GladePaletteSection *sect = gtk_object_get_data (GTK_OBJECT (elem->data), "section");
+
+      if ((sect->page == -1 && page_num == last_page)
+	  || sect->page == page_num)
+	{
+	  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (elem->data), TRUE);
+	  break;
+	}
+    }
+}
+
+
 static void
 glade_palette_init (GladePalette * palette)
 {
@@ -153,7 +180,7 @@ glade_palette_init (GladePalette * palette)
 		     pixmap);
   gtk_widget_show (pixmap);
 
-  palette->widget_button_group = gtk_radio_button_group (GTK_RADIO_BUTTON (palette->selector));
+  palette->widget_button_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (palette->selector));
   gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (palette->selector), FALSE);
   gtk_box_pack_start (GTK_BOX (hbox), palette->selector, FALSE, TRUE, 0);
   gtk_signal_connect (GTK_OBJECT (palette->selector), "toggled",
@@ -187,6 +214,9 @@ glade_palette_init (GladePalette * palette)
   gtk_notebook_set_show_border (GTK_NOTEBOOK (palette->notebook), FALSE);
   gtk_widget_show (palette->notebook);
 
+  gtk_signal_connect (GTK_OBJECT (palette->notebook), "switch_page",
+		      GTK_SIGNAL_FUNC (on_notebook_switch_page), palette);
+
   seperator = gtk_hseparator_new ();
   gtk_box_pack_end (GTK_BOX (palette->vbox), seperator, FALSE, TRUE, 3);
   gtk_widget_show (seperator);
@@ -225,7 +255,7 @@ glade_palette_add_widget (GladePalette *palette,
   button = gtk_radio_button_new (palette->widget_button_group);
   gtk_button_set_relief (GTK_BUTTON (button), relief_style);
   gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (button), FALSE);
-  palette->widget_button_group = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
+  palette->widget_button_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (button));
 
   pixmap = gtk_pixmap_new (gdkpixmap, mask);
   gtk_container_add (GTK_CONTAINER (button), pixmap);
@@ -280,6 +310,10 @@ new_section (GladePalette * palette,
 {
   GladePaletteSection *sect;
   GtkWidget *button;
+  gboolean deprecated_page = FALSE;
+
+  if (!strcmp (section, _("Dep_recated")))
+    deprecated_page = TRUE;
 
   sect = g_new (GladePaletteSection, 1);
   sect->x = sect->y = 0;
@@ -287,25 +321,34 @@ new_section (GladePalette * palette,
   gtk_widget_show (sect->table);
   sect->buttons = NULL;
   sect->buttonhash = g_hash_table_new (g_str_hash, g_str_equal);
-  gtk_notebook_append_page (GTK_NOTEBOOK (palette->notebook), sect->table,
-			    NULL);
-  sect->page = g_list_length (GTK_NOTEBOOK (palette->notebook)->children) - 1;
-  button = gtk_radio_button_new_with_mnemonic (palette->section_button_group,
-					       section);
+
+  sect->section_button = button = gtk_radio_button_new_with_mnemonic (palette->section_button_group, section);
   gtk_widget_set_name (button, section);
   gtk_widget_set_usize (button, 20, -1);
   gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (button), FALSE);
-  palette->section_button_group = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
+  palette->section_button_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (button));
   gtk_widget_show (button);
   gtk_object_set_data (GTK_OBJECT (button), "section", sect);
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
 		      GTK_SIGNAL_FUNC (on_section_button_clicked), palette);
-  /* Place the deprecated section last. */
-  if (!strcmp (section, _("Dep_recated")))
-    gtk_box_pack_end (GTK_BOX (palette->vbox), button, FALSE, TRUE, 0);
-  else
-    gtk_box_pack_start (GTK_BOX (palette->vbox), button, FALSE, TRUE, 0);
   g_hash_table_insert (palette->sections, (gpointer) section, sect);
+
+  /* Place the deprecated section last. */
+  if (deprecated_page)
+    {
+      sect->page = -1;
+      gtk_notebook_append_page (GTK_NOTEBOOK (palette->notebook), sect->table,
+				NULL);
+      gtk_box_pack_end (GTK_BOX (palette->vbox), button, FALSE, TRUE, 0);
+    }
+  else
+    {
+      sect->page = palette->next_page_to_add++;
+      gtk_notebook_insert_page (GTK_NOTEBOOK (palette->notebook), sect->table,
+				NULL, sect->page);
+      gtk_box_pack_start (GTK_BOX (palette->vbox), button, FALSE, TRUE, 0);
+    }
+
   return sect;
 }
 
@@ -430,3 +473,68 @@ glade_palette_get_widget_class (GladePalette *palette)
   return palette->selected_widget;
 }
 
+
+static void
+glade_palette_set_show_section (GladePalette        *palette,
+				GladePaletteSection *sect,
+				gboolean	     show)
+{
+  if (show)
+    gtk_widget_show (sect->section_button);
+  else
+    gtk_widget_hide (sect->section_button);
+}
+
+
+static void
+set_button_visibility (GladePaletteSection *sect,
+		       const gchar         *name,
+		       gboolean             show)
+{
+  GtkWidget *button = g_hash_table_lookup (sect->buttonhash, name);
+  if (button)
+    {
+      if (show)
+	gtk_widget_show (button);
+      else
+	gtk_widget_hide (button);
+    }
+}
+
+
+/* Shows or hides the GNOME widgets. */
+void
+glade_palette_set_show_gnome_widgets (GladePalette *palette,
+				      gboolean	    show_gnome,
+				      gboolean	    show_gnome_db)
+{
+  GladePaletteSection *sect;
+
+  /* Show or hide the Gnome page if it exists. */
+  sect = g_hash_table_lookup (palette->sections, "_Gnome");
+  if (sect)
+    glade_palette_set_show_section (palette, sect, show_gnome);
+
+  /* Show or hide the GnomeDB page if it exists. */
+  sect = g_hash_table_lookup (palette->sections, "Gnome _DB");
+  if (sect)
+    glade_palette_set_show_section (palette, sect, show_gnome_db);
+
+  /* Show or hide the GNOME widgets on the Deprecated page if they exist. */
+  sect = g_hash_table_lookup (palette->sections, "Dep_recated");
+  if (sect)
+    {
+      set_button_visibility (sect, "GnomeDialog", show_gnome);
+      set_button_visibility (sect, "GnomeMessageBox", show_gnome);
+      set_button_visibility (sect, "GnomePropertyBox", show_gnome);
+      set_button_visibility (sect, "GnomePixmap", show_gnome);
+      set_button_visibility (sect, "GnomeColorPicker", show_gnome);
+      set_button_visibility (sect, "GnomeFontPicker", show_gnome);
+      set_button_visibility (sect, "GnomeAbout", show_gnome);
+      set_button_visibility (sect, "GnomeIconList", show_gnome);
+      set_button_visibility (sect, "GnomeEntry", show_gnome);
+      set_button_visibility (sect, "GnomeFileEntry", show_gnome);
+      set_button_visibility (sect, "GnomePixmapEntry", show_gnome);
+    }
+}
+
