@@ -30,19 +30,27 @@
  */
 static GbWidget gbwidget;
 
-/* These are the special children of the widget. */
-static gchar *PropertyBoxNotebook = "GnomePropertyBox:notebook";
-
 static gchar *Title = "GnomePropertyBox|GtkWindow::title";
-static gchar *Position = "GnomePropertyBox|GtkWindow::position";
+static gchar *Position = "GnomePropertyBox|GtkWindow::window_position";
 static gchar *Modal = "GnomePropertyBox|GtkWindow::modal";
 static gchar *DefaultWidth = "GnomePropertyBox|GtkWindow::default_width";
 static gchar *DefaultHeight = "GnomePropertyBox|GtkWindow::default_height";
 static gchar *Shrink = "GnomePropertyBox|GtkWindow::allow_shrink";
 static gchar *Grow = "GnomePropertyBox|GtkWindow::allow_grow";
 static gchar *AutoShrink = "GnomePropertyBox|GtkWindow::auto_shrink";
-static gchar *WMName = "GnomePropertyBox|GtkWindow::wmclass_name";
-static gchar *WMClass = "GnomePropertyBox|GtkWindow::wmclass_class";
+static gchar *IconName = "GnomePropertyBox|GtkWindow::icon_name";
+static gchar *FocusOnMap = "GnomePropertyBox|GtkWindow::focus_on_map";
+
+static gchar *Resizable = "GnomePropertyBox|GtkWindow::resizable";
+static gchar *DestroyWithParent = "GnomePropertyBox|GtkWindow::destroy_with_parent";
+static gchar *Icon = "GnomePropertyBox|GtkWindow::icon";
+
+static gchar *Role = "GnomePropertyBox|GtkWindow::role";
+static gchar *TypeHint = "GnomePropertyBox|GtkWindow::type_hint";
+static gchar *SkipTaskbar = "GnomePropertyBox|GtkWindow::skip_taskbar_hint";
+static gchar *SkipPager = "GnomePropertyBox|GtkWindow::skip_pager_hint";
+static gchar *Decorated = "GnomePropertyBox|GtkWindow::decorated";
+static gchar *Gravity = "GnomePropertyBox|GtkWindow::gravity";
 
 static void show_gnome_property_box_dialog (GbWidgetNewData * data);
 static void on_gnome_property_box_dialog_ok (GtkWidget * widget,
@@ -74,7 +82,7 @@ gb_gnome_property_box_new (GbWidgetNewData *data)
       gb_widget_create_from (GNOME_PROPERTY_BOX (new_widget)->notebook,
 			     data->action == GB_CREATING ? "notebook" : NULL);
       gb_widget_set_child_name (GNOME_PROPERTY_BOX (new_widget)->notebook,
-				PropertyBoxNotebook);
+				GladeChildGnomePBoxNotebook);
 
       /* We connect a close signal handler which always returns TRUE so that
 	 the built-in close functionality is skipped. */
@@ -102,8 +110,8 @@ show_gnome_property_box_dialog (GbWidgetNewData * data)
   GtkObject *adjustment;
 
   dialog = glade_util_create_dialog (_("New GnomePropertyBox"), data->parent,
-				     on_gnome_property_box_dialog_ok, data,
-				     &vbox);
+				     GTK_SIGNAL_FUNC (on_gnome_property_box_dialog_ok),
+				     data, &vbox);
   gtk_signal_connect (GTK_OBJECT (dialog), "destroy",
 		      GTK_SIGNAL_FUNC (on_gnome_property_box_dialog_destroy),
 		      data);
@@ -118,9 +126,8 @@ show_gnome_property_box_dialog (GbWidgetNewData * data)
   gtk_widget_show (label);
 
   adjustment = gtk_adjustment_new (3, 1, 100, 1, 10, 10);
-  spinbutton = gtk_spin_button_new (GTK_ADJUSTMENT (adjustment), 1, 0);
-  /* save pointer to spinbutton so we can find it easily in the OK handler */
-  gtk_object_set_data (GTK_OBJECT (dialog), "pages", spinbutton);
+  spinbutton = glade_util_spin_button_new (GTK_OBJECT (dialog), "pages",
+					   GTK_ADJUSTMENT (adjustment), 1, 0);
   gtk_box_pack_start (GTK_BOX (hbox), spinbutton, TRUE, TRUE, 5);
   gtk_widget_set_usize (spinbutton, 50, -1);
   gtk_widget_grab_focus (spinbutton);
@@ -151,7 +158,7 @@ on_gnome_property_box_dialog_ok (GtkWidget * widget, GbWidgetNewData * data)
       gb_widget_create_from (GNOME_PROPERTY_BOX (new_widget)->notebook,
 			     "notebook");
       gb_widget_set_child_name (GNOME_PROPERTY_BOX (new_widget)->notebook,
-				PropertyBoxNotebook);
+				GladeChildGnomePBoxNotebook);
 
       for (i = 0; i < pages; i++)
 	{
@@ -171,6 +178,9 @@ on_gnome_property_box_dialog_ok (GtkWidget * widget, GbWidgetNewData * data)
       gtk_signal_connect (GTK_OBJECT (new_widget), "delete_event",
 			  GTK_SIGNAL_FUNC (editor_close_window), NULL);
 
+      gtk_object_set_data (GTK_OBJECT (new_widget), TypeHint,
+			   GINT_TO_POINTER (GLADE_TYPE_HINT_DIALOG_INDEX));
+
       gb_widget_initialize (new_widget, data);
       (*data->callback) (new_widget, data);
     }
@@ -194,7 +204,6 @@ gnome_property_box_new_tab_label ()
 
   label = gb_widget_new ("GtkLabel", NULL);
   g_return_val_if_fail (label != NULL, NULL);
-  gb_widget_set_child_name (label, "Notebook:tab");
   return label;
 }
 
@@ -211,7 +220,10 @@ gb_gnome_property_box_create_properties (GtkWidget * widget, GbWidgetCreateArgDa
 		  Title, NULL, Position, Modal,
 		  DefaultWidth, DefaultHeight,
 		  Shrink, Grow, AutoShrink,
-		  WMName, WMClass);
+		  IconName, FocusOnMap,
+		  Resizable, DestroyWithParent, Icon,
+		  Role, TypeHint, SkipTaskbar,
+		  SkipPager, Decorated, Gravity);
 }
 
 
@@ -229,7 +241,10 @@ gb_gnome_property_box_get_properties (GtkWidget *widget, GbWidgetGetArgData * da
 		Title, NULL, Position, Modal,
 		DefaultWidth, DefaultHeight,
 		Shrink, Grow, AutoShrink,
-		WMName, WMClass);
+		IconName, FocusOnMap,
+		Resizable, DestroyWithParent, Icon,
+		Role, TypeHint, SkipTaskbar,
+		SkipPager, Decorated, Gravity);
 }
 
 
@@ -247,7 +262,10 @@ gb_gnome_property_box_set_properties (GtkWidget * widget, GbWidgetSetArgData * d
 		Title, NULL, Position, Modal,
 		DefaultWidth, DefaultHeight,
 		Shrink, Grow, AutoShrink,
-		WMName, WMClass);
+		IconName, FocusOnMap,
+		Resizable, DestroyWithParent, Icon,
+		Role, TypeHint, SkipTaskbar,
+		SkipPager, Decorated, Gravity);
 }
 
 
@@ -289,7 +307,10 @@ gb_gnome_property_box_write_source (GtkWidget * widget, GbWidgetWriteSourceData
 				   Title, NULL, Position, Modal,
 				   DefaultWidth, DefaultHeight,
 				   Shrink, Grow, AutoShrink,
-				   WMName, WMClass);
+				   IconName, FocusOnMap,
+				   Resizable, DestroyWithParent, Icon,
+				   Role, TypeHint, SkipTaskbar,
+				   SkipPager, Decorated, Gravity);
 
   /* We output the source code for the children here, since the code should
      not include calls to create the widgets. We need to specify that the
@@ -299,7 +320,7 @@ gb_gnome_property_box_write_source (GtkWidget * widget, GbWidgetWriteSourceData
   wname = g_strdup (data->wname);
 
   source_add (data, "\n");
-  child_name = gtk_widget_get_name (GNOME_PROPERTY_BOX (widget)->notebook);
+  child_name = (char*) gtk_widget_get_name (GNOME_PROPERTY_BOX (widget)->notebook);
   child_name = source_create_valid_identifier (child_name);
   source_add (data, "  %s = GNOME_PROPERTY_BOX (%s)->notebook;\n",
 	      child_name, wname);
@@ -316,7 +337,7 @@ static GtkWidget *
 gb_gnome_property_box_get_child (GtkWidget * widget,
 				 const gchar * child_name)
 {
-  if (!strcmp (child_name, PropertyBoxNotebook))
+  if (!strcmp (child_name, GladeChildGnomePBoxNotebook))
     return GNOME_PROPERTY_BOX (widget)->notebook;
   else
     return NULL;
@@ -333,7 +354,8 @@ GbWidget*
 gb_gnome_property_box_init ()
 {
   /* Initialise the GTK type */
-  gnome_property_box_get_type();
+  volatile GtkType type;
+  type = gnome_property_box_get_type();
 
   /* Initialize the GbWidget structure */
   gb_widget_init_struct(&gbwidget);
@@ -350,6 +372,7 @@ gb_gnome_property_box_init ()
   gbwidget.gb_widget_create_properties	= gb_gnome_property_box_create_properties;
   gbwidget.gb_widget_get_properties	= gb_gnome_property_box_get_properties;
   gbwidget.gb_widget_set_properties	= gb_gnome_property_box_set_properties;
+  gbwidget.gb_widget_destroy = gb_window_destroy;
 /*
   gbwidget.gb_widget_create_popup_menu	= gb_gnome_property_box_create_popup_menu;
 */
